<?PHP
/**
 * SMF Error Log Control
 *
 * Copyright 2022 Chen Zhen @ web-develop.ca
 * This software package is distributed under the terms of its Creative Commons - Attribution No Derivatives License (by-nd) 4.0
 * License: https://creativecommons.org/licenses/by-nd/4.0/
 *
 **/
if (!defined('SMF'))
	die('<b>Error:</b> Error Log Control Failure');

function ErrorLogControl_admin_settings(&$config_vars, &$prune_toggle, $var)
{
	global $txt;

	if (!allowedTo('admin_forum'))
		fatal_lang_error('ErrorLogControl_errorMessage', false);

	$errorLogControl_config_vars = array(
		array('check', 'enableReportBans', 'help' => 'enableReportBans'),
		array('check', 'enableReportLoginAttempt', 'help' => 'enableReportLoginAttempt'),
		array('check', 'enableReportPasswordAttempt', 'help' => 'enableReportPasswordAttempt')
	);
	$key = array_search(array('check', 'enableErrorLogging'), $config_vars, false);
	$key = !empty($key) ? intval($key) : 0;
	if ($key > 0) {
		array_splice($config_vars, $key+1, 0, $errorLogControl_config_vars);
	}
	else {
		$config_vars = array_merge($errorLogControl_config_vars, $config_vars);
	}
}

function ErrorLogControl_error_types($other_error_types, $error_type, $error_message, $file, $line)
{
	global $last_error, $context, $user_info, $scripturl, $smcFunc, $sc, $db_show_debug, $txt;

	// What types of categories do we have?
	$known_error_types = array(
		'general',
		'critical',
		'database',
		'undefined_vars',
		'user',
		'ban',
		'template',
		'debug',
		'cron',
		'paidsubs',
		'backup',
		'login',
	);

	$restrictions = array(
		'cannot_access',
		'cannot_login',
		'cannot_post',
		'cannot_register',
	);

	// Collect a backtrace
	if (!isset($db_show_debug) || $db_show_debug === false)
		$backtrace = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
	else
		$backtrace = debug_backtrace();

	// Find the best query string we can...
	$query_string = empty($_SERVER['QUERY_STRING']) ? (empty($_SERVER['REQUEST_URL']) ? '' : str_replace($scripturl, '', $_SERVER['REQUEST_URL'])) : $_SERVER['QUERY_STRING'];

	// Don't log the session hash in the url twice, it's a waste.
	if (!empty($smcFunc['htmlspecialchars']))
		$query_string = $smcFunc['htmlspecialchars']((SMF == 'SSI' || SMF == 'BACKGROUND' ? '' : '?') . preg_replace(array('~;sesc=[^&;]+~', '~' . session_name() . '=' . session_id() . '[&;]~'), array(';sesc', ''), $query_string));

	// Just so we know what board error messages are from.
	if (isset($_POST['board']) && !isset($_GET['board']))
		$query_string .= ($query_string == '' ? 'board=' : ';board=') . $_POST['board'];

	// What is the true activation status of this account?
	$error_type = in_array($error_type, $known_error_types) && $error_type !== true ? $error_type : 'general';
	$error_info = array($user_info['id'], time(), $user_info['ip'], $query_string, $error_message, (string) $sc, $error_type, $file, $line, $backtrace);

	// check if we're ignoring banned entities
	if (stripos($error_type, 'user') !== FALSE && empty($modSettings['enableReportBans'])) {
		foreach ($restrictions as $restriction) {
			if (isset($_SESSION['ban'][$restriction])) {
				$last_error = $error_info;
				$context['login_errors'] = array();
				loadLanguage('Login');
				fatal_lang_error('ban_register_prohibited', false);
				die();
				redirectexit($scripturl . '?action=logout');
				break;
			}
		}
	}

	if (isset($_POST['user']))
	{
		$request = $smcFunc['db_query']('', '
			SELECT is_activated
			FROM {db_prefix}members
			WHERE ' . ($smcFunc['db_case_sensitive'] ? 'LOWER(member_name) = LOWER({string:user_name})' : 'member_name = {string:user_name}') . '
			LIMIT 1',
			array(
				'user_name' => $smcFunc['db_case_sensitive'] ? strtolower($_POST['user']) : $_POST['user'],
			)
		);
		// Probably mistyped or their email, try it as an email address. (member_name first, though!)
		if ($smcFunc['db_num_rows']($request) == 0 && strpos($_POST['user'], '@') !== false)
		{
			$smcFunc['db_free_result']($request);

			$request = $smcFunc['db_query']('', '
				SELECT is_activated
				FROM {db_prefix}members
				WHERE email_address = {string:user_name}
				LIMIT 1',
				array(
					'user_name' => $_POST['user'],
				)
			);
		}
		if ($smcFunc['db_num_rows']($request) == 0)
		{
			list($user_settings, $activation_status) = array(array(), 0);
		}
		else
		{
			$user_settings = $smcFunc['db_fetch_assoc']($request);
			$activation_status = $user_settings['is_activated'] > 10 ? $user_settings['is_activated'] - 10 : $user_settings['is_activated'];
		}

		$smcFunc['db_free_result']($request);

		//check if we're ignoring failed login attaempts or failed password attempts
		if (!empty($user_settings) && stripos($error_type, 'user') !== FALSE && empty($modSettings['enableReportLoginAttempt']) && $activation_status != 1) {
			$last_error = $error_info;
			$context['login_errors'] = array();
			loadLanguage('Login');
			fatal_lang_error('activate_not_completed1', false);
			die();
			redirectexit($scripturl . '?action=logout');
		}
		elseif (!empty($user_settings) && stripos($error_type, 'user') !== FALSE && empty($modSettings['enableReportPasswordAttempt']) && stripos($error_message, $txt['incorrect_password']) !== FALSE) {
			$last_error = $error_info;
			$context['login_errors'] = array();
			loadLanguage('Login');
			redirectexit('action=reminder');
			//fatal_lang_error('activate_not_completed1', false);
			die();
			redirectexit($scripturl . '?action=logout');
		}
	}
}

function ErrorLogControl_loadLanguage()
{
	global $txt, $helptxt;
	loadLanguage('ErrorLogControl');
}

?>